.\" $Id: ch02.ms,v 1.1.1.1 1992/03/18 09:35:47 uep Exp $
.bp
.sp
.NH 1
CREATING NEW GRAPHIC OBJECTS
.LP
This chapter contains two sections.
.br
The first one contains basic features and you have to read it if you want to
create new graphic object classes.
.br
The second one contains some more advanced features. You will be able to
build new graphic object classes even if you never read this second chapter,
but if you need memory or time optimizations, read it carefully.
.NH 2
CREATING NEW GRAPHIC OBJECT: BASIC FEATURES
.NH 3
What Must Be Done ?
.IP "Choosing the superclass"
One advantage of object oriented programming is subclassing.
This allows to share behavior and debugged and optimized code.
Therefore, if you find a nice superclass for your new object, use it!
.IP "Implementing virtual methods"
Some virtual methods must be redefined for each new class.
Through these methods the \fIGo\fP kernel will be able to manage correctly
your new object.
.IP "Implementing specific methods"
You can now write what is specific for your new class, but you must
be compatible with \fIGo\fP developing rules (coding style and pre/post
conditions).
.IP "Implementing the corresponding metaclass"
Some OO languages implement the concept of metaclass, which is a set of
informations or methods acting on every instance of the corresponding
class, but in C++, metaclasses do not exist. We have decided to
implement them. This forces you to write a metaclass object each time you
add a new class. The implementation of new metaclasses is easy,
therefore we think that advantages are greater than disadvantages.
.IP "Pre and post conditions"
Every method performing some changes on a graphic object should provide pre
and post conditions.
.LP
A concret example will be used through the following sections. We
will create the \fIGoRectangle\fP object. This will be the \fIGoRectangle\fP
you can find inside the \fIGo\fP distribution. It must be able to draw
itself either with a background or with an outline. It offers methods
to get/set its width or height.
.LP
The last (advanced) section will explain how to make pre and post condition
management faster, how common Go data are stored and how to write methods
that deal with drawings (m_draw, m_fast_draw, m_contains, m_add_clip and
m_add_opaque_clip).
.NH 3
Choosing The Superclass
.LP
A rectangle is a polygon, therefore if we choose the \fIGoPolygon\fP class
as our superclass, we will not have to implement drawings of our polygon, clip
computations, the ``contains'' method, etc.
.br
The constructor of our \fIGoRectangle\fP must initialize its superclass with
four points corresponding to a rectangle.
.nf
.cs R 20

	class GoRectangle : public GoPolygon
	{
	    ...
	};

.cs R
.fi
.NH 3
Implementing Virtual Methods
.LP
Virtual methods of the \fIGo\fP class we must redefine are:
	virtual GoMetaclass* m_metaclass () const;
	virtual void m_draw (const GRegion&) const;
	virtual void m_fast_draw (const GCoordSys&) const;
	virtual GBool m_contains (const GPoint&, GNumber maxdist) const;
	virtual void m_add_clip (GRegion&) const;
	virtual void m_add_opaque_clip (GRegion&) const;
	virtual void m_dup (const Go* orig, Go* parent, Go* prev, GBool donext);
.NH 4
virtual GoMetaclass* m_metaclass () const
.LP
This virtual method is used to get the metaclass of an anonymous object.
In our case, it must return the metaclass of \fIGoRectangle\fP.
But often, an application may need the metaclass of \fIGoRectangle\fP without
having any \fIGoRectangle\fP object. For that purpose we also have to implement
the static method:
.nf
.cs R 20

	static GoRectangleMetaclass* GoRectangle::metaclass();

.cs R
.fi
Now the virtual method is easy to write:
.nf
.cs R 20

	GoMetaclass* GoRectangle::m_metaclass () const
	{
	    return GoRectangle::metaclass();
	}

.cs R
.fi
Now we must write the static method.
.br
Each metaclass contains the list of its subclasses. This explains
following code.
.br
In fact, as for many other methods, the body of the function is the same,
you can cut and paste it and change occurences of the class name.
.br
Metaclasses instances must be unique. Also, we do not want to create them while
they are not used.
.nf
.cs R 20

	static GoRectangleMetaclass* rectangle_metaclass = NULL;

	GoRectangleMetaclass* GoRectangle::metaclass ()
	{
	    if (rectangle_metaclass == NULL)
	    {
	        rectangle_metaclass = new GoRectangleMetaclass;
	        rectangle_metaclass->superclass()->AddSubclass(rectangle_metaclass);
	    }

	    return rectangle_metaclass;
	}

.cs R
.fi
To be honest, in this method there is also this call:
.nf
.cs R 20

	GoStream::bind(GoRectangle::metaclass()->classname(),
	               (int(*)(const void*, GoStream*)) GoRectangle::save,
	               (int(*)(void**, GoStream*)) GoRectangle::restore);

.cs R
.fi
but this will be explained in the GoStream section of the advanced programers
manual.
.NH 4
virtual void m_draw (const GRegion&) const
.LP
Using graphic methods of its \fIGoInterf\fP, this method must draw the graphic
object. Graphic requests are not allowed outside of this method.
.br
This method is called by the \fIGo\fP kernel when a region intersecting with
this object has to be redrawn. Before the call, the \fIGo\fP kernel checks if
this go is not hidden by an upper go or if some upper graphic objects must
also be redrawn.
.br
Our \fIGoPolygon\fP superclass makes all drawings, therefore we do not have
to implement this method.
.NH 4
virtual void m_fast_draw (const GCoordSys&) const
.LP
This method must draw the graphic object very quickly.
.br
This method is used to have a fast real time feedback of geometry
(or any shape) changes.
.br
For instance GoUI calls this method inside some user interactions
(GoUI::move, GoUI::resize, ...).
.br
All graphic contexts used inside the m_fast_draw method must have
an ``invert'' function. This way, it is very simple to erase a previous
drawing: you simply have to call it twice.
.br
The \fIGoPolygon\fP superclass implements this method.
.NH 4
virtual GBool m_contains (const GPoint& point, GNumber maxdist) const
.LP
This method must return GTRUE if \fIpoint\fP is inside of this graphic object.
In fact if the distance between the point and drawings is less than
\fImaxdist\fP, the point is also considered inside.
.br
This method is used by \fIGoUI\fP to know precisely to wheach graphic object
a user event must be sent.
.br
The GoPolygon superclass implements it.
.NH 4
virtual void m_add_clip (GRegion&) const
.br
virtual void m_add_opaque_clip (GRegion&) const
.LP
These methods are used by the \fIGo\fP kernel to optimize management of
regions that must be redrawn.
.br
The \fIGoPolygon\fP metaclass implements them.
.NH 4
virtual void m_dup (const Go* orig, Go* parent, Go* prev, GBool donext)
.LP
The \fIGo\fP method \fIdup()\fP returns a copy of this go tree.
This method calls the virtual method ``m_dup''.
As the GoRectangle objects stores no specific data (its width and height are
retrieved from the points of the superclass), we do not have to implement
this method.
.NH 3
Implementing Specific Methods
.LP
We already know that the rectangle object must offer methods dealing with
its width and its height. We will consider these width and height as relative
to the coordinate system of this object (and not absolute, i.e. relative to
the window).
Thus a width 1 does not mean one mm, but one vector \fIcoordsys.I\fP.
(the \fIGoPolygon\fP class offers methods handling both relative or absolute
points).
.br
We have chosen to be a subclass of a polygon. A \fIGoPolygon\fP has a list of
points. There are many ways to map a rectangle to 4 points. We will choose
this way:
.RS 0.5i
.IP "first point (upper left)"
x = 0, y = 0.
.IP "second point (upper right)"
x = width, y = 0.
.IP "third point (down right)"
x = width, y = height.
.IP "fourth point (down left)"
x = 0, y = height.
.RE
.LP
We can now write methods that return width and height:
.br
(\fIpoints()\fP is a method of a \fIGoPolygon\fP which returns the array of
points)
.nf
.cs R 20

	GNumber GoRectangle::width () const
	{
	    return points()[2].x;
	}

	GNumber GoRectangle::height () const
	{
	    return points()[2].y;
	}

.cs R
.fi
We will use the polygon's method \fIreset(int nbPoint, const GPoint*)\fP to
reset width and height:
.nf
.cs R 20

	void GoRectangle::width (GNumber new_width)
	{
	    reset(new_width, -1);
	}

	void GoRectangle::height (GNumber new_height)
	{
	    reset(-1, new_height);
	}

	void GoRectangle::reset (GNumber new_w, GNumber new_h)
	{
	    if (new_w < 0) new_w = width();
	    if (new_h < 0) new_h = height();

	    GPoint pts[4];
	    pts[0] = GPoint(0, 0);
	    pts[1] = GPoint(new_w, 0);
	    pts[2] = GPoint(new_w, new_h);
	    pts[3] = GPoint(0, new_h);
	    GoPolygon::reset(4, pts);
	}

.cs R
.fi
.NH 3
Implementing The Corresponding Metaclass
.NH 4
constructor and destructor
.LP
The constructor of the \fIGoRectangleMetaclass\fP is not public. The way to get
the \fIGoRectangleMetaclass\fP is the static method
\fIGoRectangle::metaclass()\fP we have already seen. This ensure that we can
have only one instance of the \fIGoRectangleMetaclass\fP object.
.br
The destructor of the \fIGoRectangleMetaclass\fP class must:
.RS 0.5i
.IP "- destroy every instance of GoRectangle"
this is done is the GoMetaclass superclass.
.IP "- remove itself from the superclasses list of all subclasses:
.RE
.nf
.cs R 20

	GoRectangleMetaclass::~GoRectangleMetaclass ()
	{
	    if (this == rectangle_metaclass)
	    {
	        superclass()->RemoveSubclass(this);
	        rectangle_metaclass = NULL;
	    }
	}

.cs R
.fi
If a subclass of \fIGoRectangleMetaclass\fP is created, its destruction will
also call the destructor of its superclass, i.e. the destructor of
\fIGoRectangleMetaclass\fP, this is why we need the test
\fIif (this == rectangle_metaclass)\fP.
.NH 4
metaclasses virtual methods
.LP
Some virtual methods must be redefined for each new metaclass.
Through these methods, the go kernel will be able to manage correctly
your new object.
The virtual methods are:
.RS 0.5i
.IP "virtual const char*      \fBm_classname\fP () const;"
.IP "virtual GoMetaclass*     \fBm_superclass\fP () const;"
.IP "virtual const ActionRec* \fBm_actions\fP () const;"
.IP "virtual Go*              \fBm_create\fP () const;
.RE
.LP
The method \fIm_classname\fI must return the name of the graphic object:
.nf
.cs R 20

	const char* GoRectangleMetaclass::m_classname () const
	{
	    return "GoRectangle";
	}

.cs R
.fi
.LP
The method \fIm_superclass\fP must return the superclass of the metaclass:
.nf
.cs R 20

	GoMetaclass* GoRectangleMetaclass::m_superclass () const
	{
	    return GoPolygon::metaclass();
	}

.cs R
.fi
.LP
The method \fIm_actions\fP is used by \fIGoUI\fP to retrieve methods
corresponding to a string (which is found in an ascii file describing the user
interface).
.br
The \fIActionRec\fP structure is defined in \fBmGo.h\fP:
.nf
.cs R 20

	struct ActionRec
	{
	    const char* name;
	    ActionProc  proc;
	};

.cs R
.fi
Specific actions of a \fIGoRectangle\fP deal with the rectangle width and height:
.nf
.cs R 20

	static ActionRec go_rectangle_action_array[] = {
	    { "width(GNumber)",         (ActionProc)set_width },
	    { "height(GNumber)",        (ActionProc)set_height },
	    { "reset(GNumber, GNumber)", (ActionProc)set_width_height },
	    { NULL, NULL }
	};

	const ActionRec* GoRectangleMetaclass::m_actions () const
	{
	    return go_rectangle_action_array;
	}

.cs R
.fi
We will see the implementation of \fIset_width\fP, \fIset_height\fP and
\fIset_width_height\fP in the \fBImplementing Specific Methods\fP section.
.LP
The method \fIm_create\fP is used to create new instances of \fIGoRectangle\fP:
.nf
.cs R 20

	Go* GoRectangleMetaclass::m_create () const
	{
	    return (Go*)new GoRectangle;
	}

.cs R
.fi
.LP
The static method \fIGo::metaclass(const char* name)\fP returns the metaclass
of the graphic object named \fIname\fP (it uses the virtual method
\fIm_classname\fP).
.sp
The method \fIGoMetaclass::create()\fP (i.e. \fIsomeMetaclass->create()\fP)
creates and returns the graphic object which metaclass is \fIsomeMetaclass\fP
(it uses the virtual method m_create).
All this allows creation of graphic objects from a string:
.cs R 20

	Go::metaclass("GoRectangle")->create();

.cs R
.NH 3
Pre And Post Conditions
.LP
Every \fIset\fP method (i.e. a method modifying the object) should offer
pre and post conditions capabilities.
.br
There are three steps in pre and post condition implementation:

.RS 0.5i
-- Writing methods allowing the programmer to add or to remove pre or post
conditions.
-- Writing methods that call these pre or post conditions (to clarify your
code)
-- Explicitely call conditions inside the corresponding update method.
.RE

For our \fIGoRectangle\fP class, we will implement pre and post conditions on
on width and/or height modification.
.br
We have to implement these methods:
.RS 0.5i
.IP "GoRectangle::resetPrecond(...)"
.IP "GoRectangleMetaclass::resetPrecond(...)"
Methods allowing to add or to remove preconditions at instance or at
class level.
.IP "GoRectangle::resetPostcond(...)"
.IP "GoRectangleMetaclass::resetPostcond(...)"
Methods allowing to add or to remove postconditions at instance or at
class level.
.IP "GoRectangle::call_reset_precond(...)"
.IP "GoRectangleMetaclass::call_reset_precond(...)"
Private methods to call reset preconditions.
.IP "GoRectangle::call_reset_postcond(...)"
.IP "GoRectangleMetaclass::call_reset_postcond(...)"
Private method to call reset postconditions.
.RE
.NH 4
void GoRectangle::resetPrecond (...)
.LP
This method has 3 parameters:
.RS 0.5i
.IP "GBool \fBfunc\fP(void* closure, const GoRectangle*, GNumber* pNewWidth, GNumber* pNewHeight)"
The function \fIfunc\fP which will be called if someone tries to change the
width or the height of the rectangle. This method must return GFALSE if it
wants to forbid the action. If this method returns GTRUE it may update
\fI*pNewWidth\fP or \fI*pNewHeight\fP.
.IP "void* \fBclosure\fP"
This is some user data that will be passed as first parameter to the previous
function.
.IP "GAddOrRemove \fBadd_rem\fP"
If \fIadd_rem\fP is equal to \fIGADD\fP, then the precondition is added, else
(\fIGREMOVE\fP)
it is removed.
.RE
.LP
The \fIGo\fP class offers a protected method to store pre and post conditions:
.br
	AddOrRemoveConditions(key, func, closure, add_rem)
.br
\fIkey\fP must uniquely select your pre or post condition inside the program.
A good way to do it is to declare a static string inside Rectangle.cc:
.br
	static char rectangleNresetPrecond[] = "resetPrecond";
.br
the address of this string will be unique for the whole program, therefore
we can use it as a key (the contents of the string is not used at all, but
it may help for debugging).
.br
Now, we can write the method \fIresetPrecond\fP:
.nf
.cs R 20

	void GoRectangle::resetPrecond(
	        GBool func(void*, const GoRectangle*, GNumber*, GNumber*),
	        void* closure, GAddOrRemove add_rem)
	{
	    AddOrRemoveCondition(rectangleNresetPrecond, func, closure, add_rem);
	}

.cs R
.fi
.NH 4
void GoRectangleMetaclass::resetPrecond (...)
.LP
This method has the same parameters as \fIGoRectangle::resetPrecond\fP.
.br
The \fIGoMetaclass\fP class offers a method to store pre and post conditions:
\fIAddOrRemoveCondition()\fP.
.br
There is one method \fIAddOrRemoveCondition\fP defined inside the \fIGo\fP
class and used by \fIGo\fP subclasses and one method \fIAddOrRemoveCondition\fP
defined inside the \fIGoMetaclass\fP and used by \fIGoMetaclass\fP subclasses.
.br
Now, we can write the method \fIresetPrecond\fP (at metaclass level):
.nf
.cs R 20

	void GoRectangleMetaclass::resetPrecond (
	        GBool func(void*, const GoRectangle*, GNumber*, GNumber*),
	        void* closure, GAddOrRemove add_rem)
	{
	    AddOrRemoveCondition(rectangleNresetPrecond, func, closure, add_rem);
	}

.cs R
.fi
.NH 4
GBool GoRectangle::call_reset_precond (...)
.LP
Similarly to the method \fIAddOrRemoveCondition\fP the \fIGo\fP class offers
methods to call preconditions:
.br
	GBool CallPreconditions(key, parameters ...);
.br
We can then write the \fIcall_reset_precond\fP method:
.nf
.cs R 20

	GBool GoRectangle::call_reset_precond (GNumber* pW, GNumber* pH) const
	{
	    return CallPreconditions(rectangleNresetPrecond, pW, pH);
	}

.cs R
.fi
.NH 4
GBool GoRectangleMetaclass::call_reset_precond (...)
.LP
Similarly to the method \fIAddOrRemoveCondition\fP the \fIGoMetaclass\fP offers
methods to call preconditions:
.br
	GBool CallPreconditions(key, go, parameters, ...);
.br
We can then write the \fIcall_reset_precond\fP method:
.nf
.cs R 20

	GBool GoRectangleMetaclass::call_reset_precond (
	        const GoRectangle* rect, GNumber *pW, GNumber *pH)
	{
	    return CallPreconditions(rectangleNresetPrecond, rect, pW, pH);
	}

.cs R
.fi
.NH 4
void GoRectangle::resetPostcond (...)
.LP
This method has 3 parameters:
.RS 0.5i
.IP "void \fBfunc\fP(void*, GoRectangle*)"
The function \fIfunc\fP will be called after a change of the width or the
height of the rectangle.
.IP "void* \fBclosure\fP"
.IP "GAddOrRemove \fBadd_rem\fP"
As for resetPrecond.
.RE
.LP
We can now write the \fIresetPostcond\fP method:
.nf
.cs R 20

	static char rectangleNresetPostcond[] = "resetPostcond";

	void GoRectangle::resetPostcond (
	        void func(void*, GoRectangle*),
		void* closure, GAddOrRemove add_rem)
	{
	    AddOrRemoveCondition(rectangleNresetPostcond, func, closure, add_rem);
	}

.cs R
.fi
.NH 4
void GoRectangleMetaclass::resetPostcond (...)
.LP
You have enough knowledge now to write immediately the method:
.nf
.cs R 20

	void GoRectangleMetaclass::resetPostcond (
	        void func(void*, GoRectangle*),
	        void* closure, GAddOrRemove add_rem)
	{
	    AddOrRemoveCondition(rectangleNresetPostcond, func, closure, add_rem);
	}

.cs R
.fi
.NH 4
void GoRectangle::call_reset_postcond (...)
.LP
Similarly to the \fIcall_reset_precond\fP method:
.nf
.cs R 20

	void GoRectangle::call_reset_postcond ()
	{
	    CallPostconditions(rectangleNresetPostcond);
	}

.cs R
.fi
.NH 4
void GoRectangleMetaclass::call_reset_postcond (...)
.LP
.nf
.cs R 20

	void GoRectangleMetaclass::call_reset_postcond (GoRectangle* rect)
	{
	    CallPostconditions(rectangleNresetPostcond, rect);
	}

.cs R
.fi
.NH 4
modification of the reset method
.LP
We must not forget to call pre and postconditions when someone wants to
change the width or the height of our rectangle. As methods \fIwidth(GNumber)\fP
and \fIheight(GNumber)\fP call the method \fIreset(GNumber w, GNumber h)\fP we
have only to rewrite the last one.
.br
As preconditions may refuse the modification of width and/or height, its
interesting to have a reset method returning a GBool (and not void) which
will be GFALSE if the modification is not done.
.nf
.cs R 20

	GBool GoRectangle::reset (GNumber w, GNumber h)
	{
	    if (new_w < 0) new_w = width();
	    if (new_h < 0) new_h = height();

	    if (!call_reset_precond(&w, &h) ||
	        !((GoRectangleMetaclass*)getMetaclass())
	                ->call_reset_precond(this, &w, &h))
	        return GFALSE;

	    GPoint pts[4];
	    pts[0] = GPoint(0, 0);
	    pts[1] = GPoint(new_w, 0);
	    pts[2] = GPoint(new_w, new_h);
	    pts[3] = GPoint(0, new_h);
	    GoPolygon::reset(4, pts);

	    call_reset_postcond();
	    ((GoRectangleMetaclass*)GoRectangle())->call_reset_postcond(this);

	    return GTRUE;
	}

.cs R
.fi
.bp
.NH 2
CREATING NEW GRAPHIC OBJECT: ADVANCED FEATURES
.NH 3
Faster Pre And Post Conditions
.LP
The methods ``AddOrRemoveConditions'', ``CallPreconditions'' and
``CallPostconditions'' use the ``assoc'' of the go or the assoc of the
metaclass to store/retrieve ``func'' and ``closure''.
.br
An assoc uses an hash-coding to do that, therefore it is not too slow.
But if you need to be particularly fast, if you do not want to waste time
looking for, most often, inexistant pre or postconditions you can optimize
management of pre/post conditions using the following method.
.LP
We store inside the go, or inside the metaclass, a flag for each pre or post
condition, telling us if there is such a pre/post condition.
.br
If there are no pre/post conditions, the waste of time is reduced to a check
of this flag.
.br
We will reimplement methods dealing with pre/post conditions of our GoRectangle
but using such flags:
(methods ``AddOrRemoveCondition'' return GFALSE if there no condition after the
call).
.nf
.cs R 20

	class GoRectangle : public GoPolygon
	{
	    ...
	  private:
	    GBool has_reset_precond : 1;
	    GBool has_reset_postcond : 1;
	};

	GoRectangle::GoRectangle ()
	{
	    has_reset_precond = GFALSE;
	    has_reset_postcond = GFALSE;
	}

	void GoRectangle::resetPrecond (
	        GBool func(void*, const GoRectangle*, GNumber*, GNumber*),
	        void* closure, GAddOrRemove add_rem)
	{
	    has_reset_precond =
	        AddOrRemoveCondition(rectangleNresetPrecond, func, closure, add_rem);
	}

	void GoRectangle::resetPostcond (
	        void func(void*, GoRectangle*),
	        void* closure, GAddOrRemove add_rem)
	{
	    has_reset_postcond =
	        AddOrRemoveCondition(rectangleNresetPostcond, func, closure, add_rem);
	}

	GBool GoRectangle::call_reset_precond (GNumber *pW, GNumber *pH) const
	{
	    return (has_reset_precond)
	                ? CallPreconditions(rectangleNresetPrecond, pW, pH)
	                : GTRUE;
	}

	void GoRectangle::call_reset_postcond ()
	{
	    if (has_reset_postcond)
	        CallPostconditions(rectangleNresetPostcond);
	}

.cs R
.fi
And very similarly (not to say exactly) you can do the same for the metaclass
(i.e. the GoRectangleMetaclass itself has 2 flags has_reset_pre/postcond).
This optimization is particularly important for the metaclass because 
pre and postconditions are called recursively on all metaclasses (til the
metaclass which defines this pre/post condition).
.NH 3
Common Data Storage Inside Graphic Objects
.LP
Every graphic object has a parent, brothers and children, a coordinate
system, and other data. But some applications need a big GNumber of graphic
objects. For such applications, it may be very important that each graphic
object comsumes little memory.
.br
For that reason, almost no data are stored in the basic class Go. All data
accesses are done through virtual methods which name is prefixed with ``d_''
(d_get_parent, d_get_next, ...). This allows creation of low weight graphic
objects fully knowledgeble by the Go kernel.
.br
Remember that reimplementation of data storage in Go subclasses has to be done
only if you need a specific space optimization. There are already many data
storage implementations offered with the Go distribution:
.br
	GoAbsDS, GoTreeDS, GoRelDS, GoStdDS.
.br
(``DS'' means ``data storage'').
.br
Here is the list of all data storage virtual methods:
.IP "virtual Assoc* d_get_assoc () const;"

.IP "virtual Go*    d_get_parent () const;"
.IP "virtual Go*    d_get_previous () const;"
.IP "virtual Go*    d_get_next () const;"
.IP "virtual Go*    d_get_first_child () const;"
.IP "virtual GBool   d_store_children_array () const;"
.IP "virtual Go**   d_get_children (int& count) const;"
.IP "virtual void   d_set_previous (const Go*);"
.IP "virtual void   d_set_next (const Go*);"
.IP "virtual void   d_remove_child (const Go*, int child_index);"
.IP "virtual void   d_add_child (const Go*, int child_index);"
.IP "virtual void   d_set_tree (const Go* parent, const Go* prev, const Go* next);"

.IP "virtual GBool   d_store_abscs () const;"
.IP "virtual GBool   d_store_relcs () const;"
.IP "virtual const GCoordSys& d_get_relcs (const GCoordSys* parabs = NULL) const;"
.IP "virtual const GCoordSys& d_get_abscs (const GCoordSys* parabs = NULL) const;
.IP "virtual void   d_set_coordsys (const GCoordSys& relcs, const GCoordSys& abscs, const GCoordSys& parabs);"

.IP "virtual GRegion* d_get_clip () const; 
.NH 4
virtual method giving the assoc of a go
.LP
The assoc of a go is used by applications to store their data, and by the
system to store pre and post conditions.
The simplest way to implement it is:
.nf
.cs R 20

	class GoXXX : public Go
	{
	    ...
	  private:
	    GAssoc _assoc;
	};

	Assoc* GoXXX::d_get_assoc () const
	{
	    return &_assoc;
	}

.cs R
.fi
But it uses sizeof(GAssoc) bytes of memory.
Another way which uses, while the assoc is not used, only a pointer:
.br
(it may be very usefull if you need many objects, which only few of them will
need an assoc)
.nf
.cs R 20

	class GoXXX : public Go
	{
	    ...
	  private:
	    GAssoc* _assoc;
	};

	Assoc* GoXXX::d_get_assoc () const
	{
	    if (_assoc == NULL)
	        _assoc = new Assoc;
	    return _assoc;
	}

.cs R
.fi
Or, if you are sure that, inside the context of your application, the assoc
of GoXXX will never be used, it may waste zero memory:
.nf
.cs R 20

	Assoc* GoXXX::d_get_assoc () const
	{
	    return NULL;
	}

.cs R
.fi
Of course there are many other possibilities. What is important is that
a very specialized application may choose its data structure, without breaking
the Go compatibility.
.NH 4
virtual methods giving the tree structure
.LP
Graphic objects have a tree structure. Therefore they have a parent, children
and brothers.
.br
To store the parent, we do not see a better method than storing a pointer to
the parent, but getting the parent through d_get_parent makes all data access
methods more homogeneous.
.br
To store brothers or children, the Go kernel accepts two ways:
.IP "a list structure,"
.IP "an array structure."
.br
If you use a list structure, you need to implement methods d_get_previous,
d_get_next and d_get_first_child. Also your method d_store_children_array
must return GFALSE.
.br
If you use an array structure, your method d_store_children_array must
return GTRUE, and you must implement the method d_get_children (which must
return an array of Go).
.br
In all cases you must implement all set methods: d_set_previous, d_set_next,
d_remove_child, d_add_child, d_set_tree. Eventually some of these methods will
do nothing.
.br
We will now explain how set methods are called after a reparent of thisGo.
.br
d_set_previous is called on (old and new) next brother of thisGo.
.br
d_set_next is called on (old and new) previous brother of thisGo.
.br
d_remove_child is called on old parent of thisGo.
.br
d_add_child is called on new parent of thisGo.
.br
d_set_tree is called on thisGo.
.br
Therefore, if you use an array structure, you may do nothing inside methods
d_set_previous and d_set_next, because parents manage the array of children,
so it will be set up in methods d_remove_child and d_add_child.
.LP
Let us say that we prefer the list structure. The array structure is faster
only if everybody always stores the index of a go, else methods ``next'' and
``previous'' must always restart from the beginning of parent's children array.
In our experience application programmers do not like to care about such things.
.LP
An example of a very light go (concerning the tree management): only two
pointers are needed. This Go may not have children. Therefore we add a
precondition on add child for the metaclass which always return GFALSE.
.nf
.cs R 20

	class GoXXX : public Go
	{
	    ...
	  private:
	    Go* _parent;
	    Go* _next;
	};

	Go* GoXXX::d_get_parent () const
	{
	    return _parent;
	}

	Go* GoXXX::d_get_next () const
	{
	    return _next;
	}

	Go* GoXXX::d_get_previous () const
	{
	    if (_parent == NULL)
	        return NULL;

	    Go* brother = _parent->Child();
	    if (brother == this)
		return NULL;

	    while (brother->Next() != this)
	        brother = brother->Next();

	    return brother;
	}

	Go* GoXXX::d_get_first_child () const
	{
	    return NULL;
	}

	static GBool doNotAcceptAnyChild
	    (void*, const Go*, const Go*, const Go*(*))
	{
	    return GFALSE;
	}

	GoXXXMetaclass::GoXXXMetaclass ()
	{
	    ...
	    addChildPrecond(doNotAcceptAnyChild, NULL);
	}

.cs R
.fi
.NH 4
virtual methods giving the coordinate system
.LP
You have the choice between storing absolute positions and sizes (i.e. in mm
relative to the window) or positions and sizes relative to the coordinate
system of the Go parent.
If you choose to store absolute values:
	d_store_abscs must return GTRUE,
	d_store_relcs must return GFALSE,
	d_get_abscs must return the absolute coordinate system of your go.
.br
If you choose to store parent relative values:
	d_store_abscs must return GFALSE
	d_store_relcs must return GTRUE
	d_get_relsc must return the parent relative coordinate system of your go.
.br
In all cases, you must redefine:
.nf
.cs R 20

	d_set_coordsys (const GCoordSys& rel, const GCoordSys& abs,
	                const GCoordSys& parent_abs);

.cs R
.fi
.LP
An example of implementation of coordinate system management for a light
weight circle.
.br
We choose to store an absolute position, having in mind that I and J vectors
of the coordinate system are always equal to (1, 0) and (0, 1).
.br
Therefore we must add an absolute coordinate system precond which will forbid
any change of these vectors:
.nf
.cs R 20

	static GBool neverChangeIJ
	    (void*, const Go*, GCoordSys* newabs, const GCoordSys*, const GCoordSys*)
	{
	    newabs.I = Vector(1, 0);
	    newabs.J = Vector(0, 1);

	    return GTRUE;
	}
				

	GoLWCircleMetaclass::GoLWCircleMetaclass () //LW for Light Weight
	{
	    ...
	    abscsPrecond(neverChangeIJ, NULL);
	}

	class GoLWCircle : public Go
	{
	    ...
	  private:
	    GPoint _origin;
	};

	GBool GoLWCircle::d_store_abscs () const
	{
	    return GTRUE;
	}

	GBool GoLWCircle::d_store_relcs () const
	{
	    return GFALSE;
	}

	const GCoordSys& GoLWCircle::d_get_abscs (const GCoordSys*) const
	{
	    static GCoordSys cs;
	    cs.O = _origin;
	    return cs;
	}

	void GoLWCircle::d_set_coordsys
	    (const GCoordSys&, const GCoordSys& abscs, const GCoordSys&)
	{
	    _origin = abscs.O;
	}

.cs R
.fi
.NH 4
virtual method giving the clip region of the subtree
.LP
The clip of a go tree is a region, i.e. a list of boxes that contains
all drawings of this go tree. This clip is used by the Go kernel to
manage drawings of losen areas. This clip is obtained by union of clips of
each go of the tree. This operation may be expensive in time. So its a good
optimization that graphic objects, which goal is to be a parent, stores this
region (so, the kernel do not have to recompute it each time it needs it).
.br
If you do not want to lose space to store this clip your method d_get_clip
must return NULL.
Follow a possible implementation if you want to store this clip:
.nf
.cs R 20

	class GoXXX : public Go
	{
	    ...
	  private:
	    GRegion _clip;
	};

	GRegion* GoXXX::d_get_clip () const
	{
	    return &_clip;
	}

.cs R
.fi
Of course this data is only used by the Go kernel.
.NH 3
Methods That Deal With Drawings
.LP
These methods are:
.IP "virtual void m_draw (const GRegion& where) const"
When the Go kernel calls this method you must redraw yourself inside ``where''.
Your are not forbiden to draw outside of ``where'' (it is often faster to draw
everything than to check which part of the drawing is outside).
.br
This method is called by the Go kernel only if there is a not empty
intersection between ``where'' and the clip of this go.
.br
To make drawings you must use graphic methods of ``thisGo->interf()''. See
the GoInterf documentation. A simple example to draw a circle outline:
.br
(pen() is a method of this Go class returning its pen)
.nf
.cs R 20

	void GoOutlineCircle::m_draw (const GRegion& where)
	{
	    interf()->circle
	        (&_pen, NULL, Abscs(), GPoint(0, 0), _radius, 0, 2*PI);
	}

.cs R
.fi
Of course the m_draw method must draw only this go, and not its children !
The set of graphic objects that must be redrawn is managed by the Go kernel.
.IP "virtual void m_fast_draw (const GCoordSys& abs_cs) const"
This method is used (through Go::fastDraw() which calls this private virtual
method) by functions modifying the geometry of a Go through a user interaction.
.br
Therefore this method must be fast. For this reason:
	- you must use a function which inverts destination pixels. To
	clear the previous draw it is then sufficient to call a second time
	the same method (and not to manage redrawings of every graphic objects
	that were under this one).
	- you must use ``abs_cs'' as your absolute coordinate system. Because
	it may be too slow (for an interactive modification of the geometry)
	to recompute the coordinate system of each go of this go subtree.
Now an example (the same Go class than this inside the m_draw example):
.nf
.cs R 20

	void GoOutlineCircle::m_fast_draw (const GCoordSys& abs_cs)
	{
	    Pen inv_pen(_pen);
	    inv_pen.function(Ginvert);
	    interf()->circle
	        (&inv_pen, NULL, abs_cs, GPoint(0, 0), _radius, 0, 2*PI, GminQuality);
	}

.cs R
.fi
.IP "virtual void m_add_clip (GRegion& add_to_this_region) const;"
A region is a set of boxes. See the region documentation.
.br
The Go kernel uses this method when it has to know which part of the output
your go intersect.
.br
Of course the clip of a Go is only dependent of its m_draw method. For that
reason you can find inside GoInterf a method to extend a region for each
drawing method.
.br
The same example than before:
.nf
.cs R 20

	void GoOutlineCircle::m_add_clip (GRegion& add_to_this_region)
	{
	    interf()->circleClips
	       (&_pen, NULL, Abscs(), GPoint(0, 0), _radius,
	        &add_to_this_region, NULL, 0, 2*PI);
	}

.cs R
.fi
.IP "virtual void m_add_opaque_clip (GRegion&) const;"
The Go kernel uses this method to know (sometimes) when one graphic object
is on top of another one, and therefore the object which is under has nothing
to draw.
.br
Therefore if your method m_add_opaque_clip does nothing, it is correct.
If you add an opaque clip be carefull that every pixel of your opaque region
is filled by the m_draw method.
.br
The same example:
.nf
.cs R 20

	void GoOutlineCircle::m_add_opaque_clip (GRegion& add_to_this_region)
	{
	    interf()->circleClips
	        (&_pen, NULL, Abscs(), GPoint(0, 0), _radius,
	         NULL, &add_to_this_region, 0, 2*PI);
	}

.cs R
.fi
.LP
Of course the Go kernel manage drawings on damaged ares after a geometric
change (i.e. a change of a coordinate system of a go). But what must be
done by you, when your object changes ? You must never draw directly
outside of the method m_draw, and only the Go kernel is able to call this
method.
.br
What you must do (when your go changes graphically), is to declare to
the Go kernel that some areas are damaged and that (eventualy) your clip
region is updated.
.br
If the graphic change do not update the shape of your object (e.g. if your
go gets a new color), you must do the following:

	-- make your data changes to reflect the new state.

	-- tell to the kernel that your object must be redrawn
	using ``RedrawForeOnly()'' or ``RedrawBackAndFore()''.
	RedrawBackAndFore must be used if the change exposes a
	previously hidden area.
.sp
If the graphic change updates the shape of your object (e.g. if you
change the radius of a circle), you must do the following:

.RS 0.5i
-- tell to the go kernel that your current output area must be
refreshed. You do that using \fIRedrawBackAndFore()\fP.
-- make your data changes to reflect the new state.
-- tell to the kernel that your shape was updated, using \fIMyClipHasChanged()\fP.
-- tell to the kernel that you must be redrawn using \fIRedrawForeOnly()\fP.
.RE
.sp
In place of \fIRedrawBackAndFore()\fP you can use \fITempUnmap()\fP and in place
of \fIRedrawForeOnly()\fP you can use \fITempMap()\fP. Actions taken by the
\fIGo\fP kernel will then act not only on your graphic object but also on all its
subtree.
.sp
Let us see an example with a method changing the radius of a circle:
.nf
.cs R 20

	void GoOutlineCircle::radius (GNumber new_radius)
	{
	    ... calling preconditions ...

	    GBool is_mapped = mapped();

	    if (is_mapped)
	        RedrawBackAndFore();
	          //through this call, the Go kernel will know
	          //that your current output region is damaged.
	          //Therefore everything inside this region will be redrawn.

	    _radius = new_radius;
	    MyClipHasChanged();
	          //through this call, the Go kernel will know
	          //that your output region is updated.

	    if (is_mapped)
	        RedrawForeOnly();
	          //through this call, the Go kernel will know that
	          //your new output region must be redrawn.
	          //Therefore your go and every go which is on top of your go,
	          //must be redrawn.

	    ... calling postconditions ...
	}

.cs R
.fi
Of course if you implement a drawing change which does no geometry change, it
is simpler. For instance, if you change the color of your circle, you need
only:
.nf
.cs R 20

	    if (mapped())
	        RedrawForeOnly();

.cs R
.fi
.NH 3
C++ Subclassing But Not Go Subclassing
.LP
You may want to change lightly the behavior of an existing Go, but without
considering that you add a new Go class.
.br
For instance you may want to use a GoSensor which ``contains'' method checks
if the point is inside some absolute rectangle (and not, like the standard
sensor which calls its parents contains method). You have only to make
a C++ subclassing inside which you redefine the corresponding virtual method
(of course you do not have to implement a metaclass nor to redefine method
m_metaclass).
.br
loosing parts of this are:
.IP "you cannot create this special GoSensor from a string"
Go::create("GoSensor") will return a standard sensor.
.IP "you cannot save and restore your new GoSensor"
If you save and then restore it (using a GoStream) you will get an standard
sensor.
